@@ -107,6 +107,10 @@ class Kernel(Model):
     def __rand__(self, other):
         return KroneckerProduct(other, self)
 
+    @abstractmethod
+    def stationary(self) -> bool:
+        """ kernel is stationary or not """
+
 
 class Sum(Kernel):
     """ elementwise sum of two kernels """
@@ -145,9 +149,6 @@ class Sum(Kernel):
     def stationary(self) -> bool:
         return self.k1.stationary() and self.k2.stationary()
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self.k1._set(log_params[:self.b])
         self.k2._set(log_params[self.b:])
@@ -206,9 +207,6 @@ class Product(Kernel):
     def stationary(self) -> bool:
         return self.k1.stationary() and self.k2.stationary()
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self.k1._set(log_params[:self.b])
         self.k2._set(log_params[self.b:])
@@ -262,9 +260,6 @@ class Exponetiation(Kernel):
     def stationary(self) -> bool:
         return self.k.stationary()
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self.k._set(log_params)
 
@@ -317,9 +312,6 @@ class KroneckerSum(Kernel):
     def hyperparameters(self):
         return self.k1.hyperparameters + self.k2.hyperparameters
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self.k1._set(log_params[:self.b])
         self.k2._set(log_params[self.b:])
@@ -376,9 +368,6 @@ class KroneckerProduct(Kernel):
     def hyperparameters(self):
         return self.k1.hyperparameters + self.k2.hyperparameters
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self.k1._set(log_params[:self.b])
         self.k2._set(log_params[self.b:])
@@ -445,7 +434,7 @@ def check_positive_scalar_array(x: V, name='length scale'):
         raise TypeError('{} must be a float or an array.'.format(name))
 
 
-class ConstantKernel(Kernel, StationaryMixin):
+class ConstantKernel(StationaryMixin, Kernel):
     """
     constant kernel
 
@@ -476,9 +465,6 @@ class ConstantKernel(Kernel, StationaryMixin):
     def a(self):
         return exp(self.thetas.values[0])
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self._thetas.set(log_params)
 
@@ -498,7 +484,7 @@ class ConstantKernel(Kernel, StationaryMixin):
         return self.a * np.ones((X.shape[0], Z.shape[0]))
 
 
-class WhiteKernel(Kernel, StationaryMixin):
+class WhiteKernel(StationaryMixin, Kernel):
     """
     white noise kernel
 
@@ -523,9 +509,6 @@ class WhiteKernel(Kernel, StationaryMixin):
     def hyperparameters(self):
         return {}
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self._thetas.set(log_params)
 
@@ -545,7 +528,7 @@ class WhiteKernel(Kernel, StationaryMixin):
         return np.where(same, 1., 0.)
 
 
-class RBFKernel(Kernel, StationaryMixin):
+class RBFKernel(StationaryMixin, Kernel):
     """
     radial basis function kernel (a.k.a. exponential quadraric kernel)
 
@@ -588,9 +571,6 @@ class RBFKernel(Kernel, StationaryMixin):
         else:
             return np.exp(self.thetas.values)
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self._thetas.set(log_params)
 
@@ -652,7 +632,7 @@ class RBFKernel(Kernel, StationaryMixin):
         return K
 
 
-class RationalQuadraticKernel(Kernel, StationaryMixin):
+class RationalQuadraticKernel(StationaryMixin, Kernel):
 
     """
     rational quadratic kernel (scale mixture of RBF kernels)
@@ -713,9 +693,6 @@ class RationalQuadraticKernel(Kernel, StationaryMixin):
         else:
             return np.exp(self.thetas.values[1:])
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self._thetas.set(log_params)
 
@@ -777,7 +754,7 @@ class RationalQuadraticKernel(Kernel, StationaryMixin):
         return K
 
 
-class MaternKernel(Kernel, StationaryMixin):
+class MaternKernel(StationaryMixin, Kernel):
     """
     Matérn kernel (d = 1 -> Ornstein–Uhlenbeck kernel)
 
@@ -852,9 +829,6 @@ class MaternKernel(Kernel, StationaryMixin):
         else:
             return np.exp(self.thetas.values)
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self._thetas.set(log_params)
 
@@ -922,7 +896,7 @@ class MaternKernel(Kernel, StationaryMixin):
         return K
 
 
-class PeriodicKernel(Kernel, StationaryMixin):
+class PeriodicKernel(StationaryMixin, Kernel):
     """
     periodic kernel (RBF kernel in u-space)
 
@@ -984,9 +958,6 @@ class PeriodicKernel(Kernel, StationaryMixin):
         else:
             return np.exp(self.thetas.values[-self.dim:])
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self._thetas.set(log_params)
 
@@ -1056,8 +1027,9 @@ class PeriodicKernel(Kernel, StationaryMixin):
         K = np.exp(R2_l2 / -2.)
         return K
 
+# FIXME: add CosineKernel, spectral kernel = RBFKernel * CosineKernel?
 
-class SpectralKernel(Kernel, StationaryMixin):
+class SpectralKernel(StationaryMixin, Kernel):
     """
     additive component of spectral mixture kernel
     whose spectral densitiy is Gaussian(p, l)
@@ -1120,9 +1092,6 @@ class SpectralKernel(Kernel, StationaryMixin):
         else:
             return np.exp(self.thetas.values[-self.dim:])
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self._thetas.set(log_params)
 
@@ -1201,12 +1170,12 @@ class SpectralKernel(Kernel, StationaryMixin):
         return K
 
 
-class LinearKernel(Kernel, NonStationaryMixin):
+class LinearKernel(NonStationaryMixin, Kernel):
     """
     linear kernel
 
     k(x,z) = (x/l).T @ (z/l)
-
+    # FIXME: reparametrise ((x-c)/l).T @ ((z-c)/l)?
     l (length scale): positive float (isotropic) or positive array (anisotropic)
     """
 
@@ -1244,9 +1213,6 @@ class LinearKernel(Kernel, NonStationaryMixin):
         else:
             return np.exp(self.thetas.values)
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self._thetas.set(log_params)
 
@@ -1297,7 +1263,7 @@ class LinearKernel(Kernel, NonStationaryMixin):
         return K
 
 
-class NeuralKernel(Kernel, NonStationaryMixin):
+class NeuralKernel(NonStationaryMixin, Kernel):
     """
     neural kernel
 
@@ -1348,9 +1314,6 @@ class NeuralKernel(Kernel, NonStationaryMixin):
         else:
             return np.exp(self.thetas.values[1:])
 
-    def fitted(self) -> bool:
-        return self.thetas.assigned()
-
     def _set(self, log_params: ndarray):
         self._thetas.set(log_params)
 
